<?php

// -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
/**
 *   Paloose: Cocoon without Java
 *
 *   Paloose is a Cocoon variant written entirely in PHP without Java.
 *   It resulted from scratching a long standing personal itch: that there are very few
 *   ISPs who will support Java/Tomcat for web sites, other than as a very expensive
 *   "professional" addition. Almost all will support PHP5 (sorry, Paloose does not use PHP4)
 *   and so I decided to write my version of a simple, cut-down Cocoon in PHP5. I have been using
 *   Cocoon for some time now and have always found it the best way to build XML/XSL
 *   based sites - is there any other way? :-)
 *   
 *   The code that builds Paloose is a little rough and ready in places. This is my
 *   first PHP program (other than the odd embedded line within HTML) and it shows. I am
 *   more used to program in strongly typed languages having had a lifetime using
 *   Algol, Coral, Algol68, Pascal, Java at al. I have programmed in Perl for many
 *   years and so PHP was not a wholly new experience. I am sure that I have used 10 lines
 *   in places where PHP5 would let me use one; never mind - it suffices, the future will
 *   refine the code.
 *   
 *   "<i>The villiany you teach me I will execute, and it will go hard
 *    but I will better the instruction.</i>" William Shakespeare, The Merchant of Venice Act III Scene 1
 *
 * LICENSE:
 * 
 * Paloose is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program.  If not, see <http://www.gnu.org/licenses/> 
 *
 * @package paloose
 * @author Hugh Field-Richards <hsfr@hsfr.org.uk>
 * @version 1.3.5b5
 * @license http://www.opensource.org/licenses/lgpl-license.php LGPL
 * @copyright 2006 - 2010 Hugh Field-Richards
 * 
 * For version history see associated VERSION file
 */

$_SERVER[ 'DOCUMENT_ROOT' ] = substr( $_SERVER[ 'SCRIPT_FILENAME'], 0, strrpos( $_SERVER[ 'SCRIPT_FILENAME'], "/" ) + 1 );

// error_reporting( E_ERROR );      // Only output errors at present - mature system should only report through Paloose exceptions :-)
ini_set( 'track_errors', '1' );

//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
// Do not change anything in this file. All changes are done in the calling
// file (paloose.php) in your web site folder which sets PALOOSE_DIRECTORY.

/**  @ignore  */
define( 'PALOOSE_DIR', $_SERVER[ 'DOCUMENT_ROOT' ] . PALOOSE_DIRECTORY );

/**  @ignore  */

define( 'PHP_DIR', PALOOSE_DIR . '/lib' );

//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
// Will need this to resolve all pseudo-protocols below
require_once( PHP_DIR . "/environment/StringResolver.php" );

// Prime the $gConfiguration array with user requirements
require_once( PHP_DIR . "/config.inc.php" );

// Added for PHP V5.3.0
date_default_timezone_set( $gConfiguration[ 'timeZone' ] );

//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
// The environment class is a useful means of providing context information and
// common useful methods for the environment processing.
require_once( PHP_DIR . "/environment/Environment.php" );
require_once( PHP_DIR . "/environment/Utilities.php" );
Environment::$gQueryString = $_SERVER[ 'QUERY_STRING' ];

//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
// Prime the logging system (using log4php). Must be done outside any class
// to make it available to all defined classes. The default directory is to be
// the one included with the distribution. If you want to put log4php in
// another directory (you may already use it) define it in your "paloose.php"
// file in your site's root directory (see documentation).

/**  @ignore  */
if ( !defined( 'LOG4PHP_DIR' ) ) define( 'LOG4PHP_DIR', PHP_DIR . '/log4php' );

include_once( LOG4PHP_DIR . '/Logger.php' );
Logger::configure( StringResolver::expandPseudoProtocols( $gConfiguration[ 'loggerConfig' ] ) );
$Logger = Logger::getLogger( 'main' );

$Logger->info( '==================================================');
$Logger->info( 'Raw Request URI: ' . $_SERVER[ 'REQUEST_URI' ] );
$Logger->info( 'Raw Query string: ' . $_SERVER[ 'QUERY_STRING' ] );
$Logger->info( 'Remote address: ' . $_SERVER[ 'REMOTE_ADDR' ] );
$Logger->debug( 'DOCUMENT_ROOT: ' . $_SERVER[ 'DOCUMENT_ROOT' ] );
$Logger->debug( 'PALOOSE_DIR: ' . PALOOSE_DIR );
$Logger->debug( 'PHP_DIR: ' . PHP_DIR );
$Logger->debug( 'LOG4PHP_DIR: ' . LOG4PHP_DIR );
$Logger->debug( 'loggerConfig: ' . StringResolver::expandPseudoProtocols( $gConfiguration[ 'loggerConfig' ] ) );

//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
// Set up the various Input Modules. These are stored in a single list
// rather than a lot of separate instances which would each be required to
// be made global.
$gModules = array();

require_once( PHP_DIR . "/environment/RequestParameterModule.php" );
$requestParameterModule = new RequestParameterModule( 'request-param' );
$gModules[ $requestParameterModule->getName() ] = $requestParameterModule;
// Must be done before sessions are set up
$requestParameterModule->parseQuery( $_SERVER[ 'QUERY_STRING' ] );

require_once( PHP_DIR . "/environment/FlowModule.php" );
$flowModule = new FlowModule( 'flow' );
$gModules[ $flowModule->getName() ] = $flowModule;

require_once( PHP_DIR . "/environment/ParameterModule.php" );
$paramsModule = new ParameterModule( 'params' );
$gModules[ $paramsModule->getName() ] = $paramsModule;

require_once( PHP_DIR . "/environment/GlobalModule.php" );
$globalsModule = new ParameterModule( 'global' );
$gModules[ $globalsModule->getName() ] = $globalsModule;

//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
// The sitemap is the main repository of all that happens in Paloose.
// The master sitemap is the root of the stack and sub-sitemaps are pushed
// on the stack as required.
require_once( PHP_DIR . "/sitemap/Sitemap.php" );
require_once( PHP_DIR . "/sitemap/SitemapStack.php" );
$gSitemapStack = new SitemapStack();

//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
// Create the session handler instance even though we may not need it.
// We have to set up the session module first.
ini_set( 'arg_separator.output','&amp;' );
require_once( PHP_DIR . "/environment/SessionModule.php" );
$sessionModule = new SessionModule( 'session' );
$gModules[ $sessionModule->getName() ] = $sessionModule;

require_once( PHP_DIR . "/environment/Sessions.php" );
$gSessionHandler = new Sessions();

//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
// Each level of match has its own associated variables. These are stored as arrays
// on the variable stack.
require_once( PHP_DIR . "/sitemap/VariableStack.php" );
$gVariableStack = new VariableStack();

require_once( PHP_DIR . "/serialization/OutputStream.php" );
require_once( PHP_DIR . "/exception-handling/PalooseException.php" );
require_once( PHP_DIR . "/exception-handling/InternalException.php" );
require_once( PHP_DIR . "/exception-handling/UserException.php" );
require_once( PHP_DIR . "/exception-handling/RunException.php" );
require_once( PHP_DIR . "/exception-handling/ExitException.php" );
require_once( PHP_DIR . "/error-handling/ErrorPage.php" );

// Where all the work is done. We do not emerge from here until everything is finished.
$paloose = new Paloose();

// All done - so clean up and exit
Logger::shutdown();
exit;

//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
/**
 * The main Paloose class where everything happens.
 * 
 * This is run when the http request
 * is redirected here, courtesy of the .htaccess file in the user's site home folder.
 * It is called from the paloose.php file that is in the user's site folder, that
 * paloose.php file remains the only means to configure Paloose.
 *
 * @package paloose
 * @subpackage Paloose
 */
 
class Paloose {

   /** Logger instance for this class */   
   private $gLogger;
   
   /** Root sitemap */   
   private $gSitemap;

   //-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
   //-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
   /**
    * The constructor for the Paloose class.
    * 
    * This contains the entire functionality for
    * Paloose. It sets up all the required variables (global and local), initialises
    * the logger, parses and runs the root sitemap. All exception errors and early exit
    * track back to here.
    */
    
   function __construct()
   {
      global $gSitemapStack;
      global $gConfiguration;
      global $gModules;
      
      $rootSitemapDir = $_SERVER[ 'DOCUMENT_ROOT'];
      // Remove trailing path separator if required (all directory names are stored without
      // separator, which is put in as required when used).
      if ( substr( $rootSitemapDir, strlen( $rootSitemapDir ) - 1, 1 ) == Environment::getPathSeparator() ) {
         $rootSitemapDir = substr( $rootSitemapDir, 0, strlen( $rootSitemapDir ) - 1 );
      }

      $this->gLogger = Logger::getLogger( 'Paloose' );

      // Correct the URI and Query string to that requested. This is done
      // because of the redirect in the .htaccess file. Change that way of
      // doing things and you will have to change this routine.
      $matchableURI = Utilities::correctURI();

      $this->gLogger->debug( "Actual URI : " . $_SERVER[ 'REQUEST_URI' ] );
      $this->gLogger->debug( "Actual Query string : " . $_SERVER[ 'QUERY_STRING' ] );
      $this->gLogger->debug( "Matchable URI : " . $matchableURI );

      $requestParameterModule = $gModules[ 'request-param' ];
      // $requestParameterModule->parseQuery( $_SERVER[ 'QUERY_STRING' ] );
      $requestParameterModule->add( "resource", $matchableURI );

      // All errors and early exit exceptions come back to here
      try {
         // To start with the output stream is the standard one to reply to the client
         ob_start();
         $outputStream = new OutputStream( OutputStream::STANDARD_OUTPUT );

         // Set up the root sitemap, parse it and run it
         Environment::$gRootSitemapDir = $rootSitemapDir;
         $this->gSitemap = new Sitemap( $rootSitemapDir, $gConfiguration[ 'rootSiteMap' ], NULL, $outputStream );
         $gSitemapStack->push( $this->gSitemap );
         $this->gLogger->debug( "Parse sitemaps ----------------------------------" );
         $this->gSitemap->parse();
         $this->gLogger->debug( "Running sitemaps ----------------------------------" );
         // Not an internal request so mark false
         $this->gSitemap->run( $matchableURI, $_SERVER[ 'QUERY_STRING' ], false );
         $gSitemapStack->pop();
      } catch ( ExitException $e ) {
         // It is a little suspect to use exceptions to handle what is really
         // a normal exit (albeit an early exit), however it works quite nicely.
         $this->gLogger->debug( "Clean exit" );
         // Clean exit so finish
      } catch( InternalException $e ) {
         $this->gLogger->fatal( $e->getMessage() );
         $errorPage = new ErrorPage( $e->getMessage(), $e->getCode(), NULL );
         $errorPage->display();
      } catch( UserException $e ) {
         $this->gLogger->fatal( $e->getMessage() );
         $errorPage = new ErrorPage( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
         $errorPage->display();
      } catch( RunException $e ) {
         $this->gLogger->fatal( $e->getMessage() );
         $errorPage = new ErrorPage( $e->getMessage(), $e->getCode(), NULL );
         $errorPage->display();
      }
      // Flush the output buffer to the client before exiting (do not clean buffer afterwards!!)
      ob_end_flush();
   }

}
?>